#!/usr/bin/python3
#  coding: utf-8
#
#   calcSectionCoeffDDialog.py
#
#       beamの断面形状から断面の定数を計算する。
#       断面積、断面2次モーメント、ねじり定数
#
#   21/02/17    新規作成
#      02/25    断面系数Zyy, Zzzの計算を追加
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

import os, sys
import pyFistr
import GtkParts
import math

ls = "\n"

#---------------------
#  dialogApp class
#---------------------
class dialogApp:

    def __init__(self):
        self.builder = Gtk.Builder()
        path = os.getenv("easyIstrPath") + os.sep + "python" + os.sep
        self.builder.add_from_file(path + "calcSectionCoeffDDialog.glade")
        self.dialog = self.builder.get_object("dialog1")
        self.dialog.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.setGtkObject()
        #変数を定義
        self.stat = []
        self.comboItems = (_("丸棒"), _("パイプ"), _("角棒"), _("中空角棒"), _("H型"), _("I型"), _("L型"))
        self.materials = ("round", "pipe", "square", "squarePipe", "H", "I", "L")
        self.resItems = ("material", "area", "Iyy", "Izz", "Jx", "Zyy", "Zzz")
        self.settingFile = os.sep.join([
            os.getenv("easyIstrUserPath"), 
            "data", 
            "calcSectionCoeffDDialog_data"])
        #初期化
        self.initialize()

    #
    #  show
    #--------
    def show(self):
        self.dialog.run()
        return self.stat

    #
    #  close
    #---------
    def close(self, *args):
        settingDict = self.getSettingDataFromGui()
        self.saveSettingDataToFile(settingDict)
        self.dialog.destroy()

    #
    #  setGtkObject
    #---------------
    def setGtkObject(self):
        self.entry_area = self.builder.get_object("entry_area")
        self.entry_Iyy = self.builder.get_object("entry_Iyy")
        self.entry_Izz = self.builder.get_object("entry_Izz")
        self.entry_Jx = self.builder.get_object("entry_Jx")
        self.entryMaru_dia = self.builder.get_object("entryMaru_dia")
        self.entryPipe_d = self.builder.get_object("entryPipe_d")
        self.entryPipe_t = self.builder.get_object("entryPipe_t")
        self.entryKaku_B = self.builder.get_object("entryKaku_B")
        self.entryKaku_h = self.builder.get_object("entryKaku_h")
        self.entryChuKaku_B = self.builder.get_object("entryChuKaku_B")
        self.entryChuKaku_h = self.builder.get_object("entryChuKaku_h")
        self.entryChuKaku_t = self.builder.get_object("entryChuKaku_t")
        self.entryH_B = self.builder.get_object("entryH_B")
        self.entryH_h = self.builder.get_object("entryH_h")
        self.entryH_t1 = self.builder.get_object("entryH_t1")
        self.entryH_t2 = self.builder.get_object("entryH_t2")
        self.entryI_B = self.builder.get_object("entryI_B")
        self.entryI_h = self.builder.get_object("entryI_h")
        self.entryI_t1 = self.builder.get_object("entryI_t1")
        self.entryI_t2 = self.builder.get_object("entryI_t2")
        self.entryL_B = self.builder.get_object("entryL_B")
        self.entryL_h = self.builder.get_object("entryL_h")
        self.entryL_t1 = self.builder.get_object("entryL_t1")
        self.entryL_t2 = self.builder.get_object("entryL_t2")
        self.label_Zyy = self.builder.get_object("label_Zyy")
        self.label_Zzz = self.builder.get_object("label_Zzz")
        self.note_mat = self.builder.get_object("note_mat")
        combo_mat = self.builder.get_object("combo_mat")
        self.combo_mat = GtkParts.comboBox(combo_mat)

    #
    #  initialize
    #-------------
    def initialize(self):
        """ 初期化"""
        #comboにデータをセット(初期値)
        items = self.comboItems
        self.combo_mat.setItems(items)
        pageNo = 0
        self.combo_mat.selectItem(items[pageNo])
        self.note_mat.set_current_page(pageNo)
        #settingFileからデータを取得してセット
        settingDict = self.getSettingDataFromFile()
        self.setSettingDataToGui(settingDict)

    #------------ event handler --------------
    def onChangeMat(self, event):           #材料選択時
        self.changePage()
    def onMaruCalc(self, event):            #丸棒計算
        self.calcMaru()
    def onPipeCalc(self, event):            #パイプ計算
        self.calcPipe()
    def onKakuCalc(self, event):            #角棒計算
        self.calcKaku()
    def onChuKakuCalc(self, event):         #中空角棒
        self.calcChuKaku()
    def onHCalc(self, event):               #H型
        self.calcH()
    def onICalc(self, event):               #I型
        self.calcI()
    def onLCalc(self, event):               #L型
        self.calcL()
    def onCancel(self, event):              #キャンセル
        self.stat = []
        self.close()
    def onOk(self, event):                  #適用
        self.stat = self.getCalcResult()
        self.close()
    #-----------------------------------------

    #
    #  changePage
    #-------------
    def changePage(self):
        item = self.combo_mat.getSelectedItem()
        pageNo = self.comboItems.index(item)
        self.note_mat.set_current_page(pageNo)

    #
    #  calcMaru
    #-----------
    def calcMaru(self):
        """ 丸棒の計算"""
        textDia = self.entryMaru_dia.get_text()
        try:
            dia = float(textDia)
        except:
            title = _("エラー")
            mess = _("丸棒の外径が取得できません。") + ls
            mess += _("外形を入力し直してください。")
            self.errDialog(title, mess)
            return
        area = math.pi * dia**2 / 4.0
        Izz = math.pi * dia**4 / 64.0
        Iyy = Izz
        Jx = Iyy + Izz
        self.setCoeffToEntry(area, Iyy, Izz, Jx)
        Zyy = Iyy / (dia / 2.0)
        Zzz = Zyy
        self.setCoeffToLabel(Zyy, Zzz)

    #
    #  setCoeffToEntry
    def setCoeffToEntry(self, area, Iyy, Izz, Jx):
        """ 定数をentryにセットする"""
        strArea = "%11.5e" % area
        strIyy = "%11.5e" % Iyy
        strIzz = "%11.5e" % Izz
        strJx = "%11.5e" % Jx
        self.entry_area.set_text(strArea)
        self.entry_Iyy.set_text(strIyy)
        self.entry_Izz.set_text(strIzz)
        self.entry_Jx.set_text(strJx)

    #
    #  setCoeffTolabel
    def setCoeffToLabel(self, Zyy, Zzz):
        """ 断面系数をlabelにセット"""
        strZyy = "%11.5e" % Zyy
        strZzz = "%11.5e" % Zzz
        self.label_Zyy.set_text(strZyy)
        self.label_Zzz.set_text(strZzz)

    #
    #  calcPipe
    #------------
    def calcPipe(self):
        """ パイプの計算"""
        text_d = self.entryPipe_d.get_text()
        text_t = self.entryPipe_t.get_text()
        try:
            d = float(text_d)
            t = float(text_t)
        except:
            title = _("エラー")
            mess = _("パイプの外径 or 厚さが取得できません。") + ls
            mess += _("入力し直してください。")
            self.errDialog(title, mess)
            return
        d2 = d - (2 * t)
        area = math.pi * (d**2 - d2**2) / 4.0
        Izz = math.pi * (d**4 - d2**4) / 64.0
        Iyy = Izz
        Jx = Iyy + Izz
        self.setCoeffToEntry(area, Iyy, Izz, Jx)
        Zzz = Izz / (d / 2.0)
        Zyy = Iyy / (d / 2.0)
        self.setCoeffToLabel(Zyy, Zzz)

    #
    #  calcKaku
    #-----------
    def calcKaku(self):
        """ 角棒の計算"""
        text_B = self.entryKaku_B.get_text()
        text_h = self.entryKaku_h.get_text()
        try:
            B = float(text_B)
            h = float(text_h)
        except:
            title = _("エラー")
            mess = _("角棒の定数が取得できません。") + ls
            mess += _("入力し直してください。")
            self.errDialog(title, mess)
            return
        area = B * h
        Izz = B * h**3 / 12.0
        Iyy = h * B**3 / 12.0
        Jx = Iyy + Izz
        self.setCoeffToEntry(area, Iyy, Izz, Jx)
        Zzz = Izz / (h / 2.0)
        Zyy = Iyy / (B / 2.0)
        self.setCoeffToLabel(Zyy, Zzz)

    #
    #  calcChuKaku
    #--------------
    def calcChuKaku(self):
        """ 中空角棒の計算"""
        text_B = self.entryChuKaku_B.get_text()
        text_h = self.entryChuKaku_h.get_text()
        text_t = self.entryChuKaku_t.get_text()
        try:
            B = float(text_B)
            h = float(text_h)
            t = float(text_t)
        except:
            title = _("エラー")
            mess = _("中空角棒の定数が取得できません。") + ls
            mess += _("入力し直してください。")
            self.errDialog(title, mess)
            return
        B2 = B - 2.0 * t
        h2 = h - 2.0 * t
        area = (B * h) - (B2 * h2)
        Izz = B * h**3 / 12.0 - B2 * h2**3 / 12.0
        Iyy = h * B**3 / 12.0 - h2 * B2**3 / 12.0
        Jx = Iyy + Izz
        self.setCoeffToEntry(area, Iyy, Izz, Jx)
        Zzz = Izz / (h / 2.0)
        Zyy = Iyy / (B / 2.0)
        self.setCoeffToLabel(Zyy, Zzz)

    #
    #  calcH
    #--------
    def calcH(self):
        """ H型の計算"""
        text_B = self.entryH_B.get_text()
        text_h = self.entryH_h.get_text()
        text_t1 = self.entryH_t1.get_text()
        text_t2 = self.entryH_t2.get_text()
        try:
            B = float(text_B)
            h = float(text_h)
            t1 = float(text_t1)
            t2 = float(text_t2)
        except:
            title = _("エラー")
            mess = _("H型の定数が取得できません。") + ls
            mess += _("入力し直してください。")
            self.errDialog(title, mess)
            return
        area = B * h - (B - 2.0 * t2) * (h - t1)
        Izz = (B * t1**3 / 12.0) + (t2 * h**3 / 6.0) - (t2 * t1**3 / 6.0)
        Iyy = h * B**3 / 12.0 - ((h-t1) * (B - 2.0*t2)**3) / 12.0
        Jx = Iyy + Izz
        self.setCoeffToEntry(area, Iyy, Izz, Jx)
        Zzz = Izz / (h / 2.0)
        Zyy = Iyy / (B / 2.0)
        self.setCoeffToLabel(Zyy, Zzz)

    #
    #  calcI
    #--------
    def calcI(self):
        """ I型鋼の計算"""
        text_B = self.entryI_B.get_text()
        text_h = self.entryI_h.get_text()
        text_t1 = self.entryI_t1.get_text()
        text_t2 = self.entryI_t2.get_text()
        try:
            B = float(text_B)
            h = float(text_h)
            t1 = float(text_t1)
            t2 = float(text_t2)
        except:
            title = _("エラー")
            mess = _("I型の定数が取得できません。") + ls
            mess += _("入力し直してください。")
            self.errDialog(title, mess)
            return
        area = h * B - (h - 2.0 * t2) * (B - t1)
        Izz = B * h**3 / 12.0 - ((B-t1) * (h - 2.0*t2)**3) / 12.0
        Iyy = (h * t1**3 / 12.0) + (t2 * B**3 / 6.0) - (t2 * t1**3 / 6.0)
        Jx = Iyy + Izz
        self.setCoeffToEntry(area, Iyy, Izz, Jx)
        Zzz = Izz / (h / 2.0)
        Zyy = Iyy / (B / 2.0)
        self.setCoeffToLabel(Zyy, Zzz)

    #
    #  calcL
    #--------
    def calcL(self):
        """ L型鋼の計算"""
        text_B = self.entryL_B.get_text()
        text_h = self.entryL_h.get_text()
        text_t1 = self.entryL_t1.get_text()
        text_t2 = self.entryL_t2.get_text()
        try:
            B = float(text_B)
            h = float(text_h)
            t1 = float(text_t1)
            t2 = float(text_t2)
        except:
            title = _("エラー")
            mess = _("L型の定数が取得できません。") + ls
            mess += _("入力し直してください。")
            self.errDialog(title, mess)
            return
        area = B * h - (B - t1) * (h - t2)
        C1 = h**2 * t1 + (B - t1) * t2**2
        C2 = 2.0 * (h * t1 + (B - t1) * t2)
        e1 = h - C1 / C2
        e2 = C1 / C2
        Izz = 1.0 / 3.0 * (B * e2**3 - (B - t1) * (e2 - t2)**3 + t2 * e1**3)
        Zzz = Izz / e1
        #B, Hを入れ替え
        B, h = h, B
        C1 = h**2 * t1 + (B - t1) * t2**2
        C2 = 2.0 * (h * t1 + (B - t1) * t2)
        e1 = h - C1 / C2
        e2 = C1 / C2
        Iyy = 1.0 / 3.0 * (B * e2**3 - (B - t1) * (e2 - t2)**3 + t2 * e1**3)
        Jx = Iyy + Izz
        Zyy = Iyy / e1
        self.setCoeffToEntry(area, Iyy, Izz, Jx)
        self.setCoeffToLabel(Zyy, Zzz)

    #
    #  getCalcResult
    #----------------
    def getCalcResult(self):
        """ 計算結果を取得して戻す"""
        area = self.entry_area.get_text()
        Iyy = self.entry_Iyy.get_text()
        Izz = self.entry_Izz.get_text()
        Jx = self.entry_Jx.get_text()
        return [area, Iyy, Izz, Jx]
    
    #
    #  getSettingDataFromGui
    #------------------------
    def getSettingDataFromGui(self):
        """ settingDataをGUIから取得する"""
        settingDict = {}
        #設定材料を取得
        matName = self.combo_mat.getSelectedItem()
        idx = self.comboItems.index(matName)
        selMat = self.materials[idx]
        settingDict["material"] = selMat
        #計算結果を取得
        area = self.entry_area.get_text()
        Iyy = self.entry_Iyy.get_text()
        Izz = self.entry_Izz.get_text()
        Jx = self.entry_Jx.get_text()
        Zyy = self.label_Zyy.get_text()
        Zzz = self.label_Zzz.get_text()
        settingDict["area"] = area
        settingDict["Iyy"] = Iyy
        settingDict["Izz"] = Izz
        settingDict["Jx"] = Jx
        settingDict["Zyy"] = Zyy
        settingDict["Zzz"] = Zzz
        #丸棒を取得
        d = self.entryMaru_dia.get_text()
        settingDict["round"] = {}
        settingDict["round"]["d"] = d
        #pipe
        d = self.entryPipe_d.get_text()
        t = self.entryPipe_t.get_text()
        settingDict["pipe"] = {}
        settingDict["pipe"]["d"] = d
        settingDict["pipe"]["t"] = t
        #角棒を取得
        B = self.entryKaku_B.get_text()
        h = self.entryKaku_h.get_text()
        settingDict["square"] = {}
        settingDict["square"]["B"] = B
        settingDict["square"]["h"] = h
        #中空角棒
        B = self.entryChuKaku_B.get_text()
        h = self.entryChuKaku_h.get_text()
        t = self.entryChuKaku_t.get_text()
        settingDict["squarePipe"] = {}
        settingDict["squarePipe"]["B"] = B
        settingDict["squarePipe"]["h"] = h
        settingDict["squarePipe"]["t"] = t
        #H型
        B = self.entryH_B.get_text()
        h = self.entryH_h.get_text()
        t1 = self.entryH_t1.get_text()
        t2 = self.entryH_t2.get_text()
        settingDict["H"] = {}
        settingDict["H"]["B"] = B
        settingDict["H"]["h"] = h
        settingDict["H"]["t1"] = t1
        settingDict["H"]["t2"] = t2
        #I型
        B = self.entryI_B.get_text()
        h = self.entryI_h.get_text()
        t1 = self.entryI_t1.get_text()
        t2 = self.entryI_t2.get_text()
        settingDict["I"] = {}
        settingDict["I"]["B"] = B
        settingDict["I"]["h"] = h
        settingDict["I"]["t1"] = t1
        settingDict["I"]["t2"] = t2
        #L型
        B = self.entryL_B.get_text()
        h = self.entryL_h.get_text()
        t1 = self.entryL_t1.get_text()
        t2 = self.entryL_t2.get_text()
        settingDict["L"] = {}
        settingDict["L"]["B"] = B
        settingDict["L"]["h"] = h
        settingDict["L"]["t1"] = t1
        settingDict["L"]["t2"] = t2
        return settingDict

    #
    #  getSettingDataFromFile
    #-------------------------
    def getSettingDataFromFile(self):
        """ fileからsettingDataを取得する"""
        settingDict = {}
        fileName = self.settingFile
        if os.path.exists(fileName) == False:
            return settingDict

        resKeys = self.resItems
        f = open(self.settingFile, encoding="utf-8"); lines = f.readlines(); f.close()
        lines = pyFistr.deleteComment(lines)
        for line in lines:
            if line[0] != " ":
                words = line.split()
                if len(words) >= 1:
                    key = words[0]
                    if key in resKeys:
                        val = ""
                        if len(words) >= 2:
                            val = words[1]
                        settingDict[key] = val
                    else:
                        settingDict[key] = {}
            else:
                words = line.split()
                if len(words) >= 1:
                    subKey = words[0]
                    val = ""
                    if len(words) >= 2:
                        val = words[1]
                    settingDict[key][subKey] = val
        return settingDict

    #
    #  saveSettingDataToFile
    #-------------------------
    def saveSettingDataToFile(self, settingDict):
        """ settingDataをfileに保存"""
        #file内容を作成
        resLines = []
        lines = []
        lines += ["#\n"]
        lines += ["#  calcSectionCoeffDDialog setting data\n"]
        lines += ["#\n"]
        lines += ["\n"]
        for key in settingDict.keys():
            if type(settingDict[key]) == dict:
                lines += [key + "\n"]
                for subKey in settingDict[key].keys():
                    lines += ["    " + subKey + " " + settingDict[key][subKey] + "\n"]
            elif type(settingDict[key]) == str:
                resLines += [key + " " + settingDict[key] + "\n"]
        lines += resLines
        #書き込み
        f = open(self.settingFile, "w", encoding="utf-8")
        f.writelines(lines)
        f.close()

    #
    #  setSettingDataToGui
    #-----------------------
    def setSettingDataToGui(self, settingDict):
        """ settingDataをGUIにセットする"""
        errFlag = 0
        #comboを設定
        try:
            mat = settingDict["material"]
        except:
            errFlag = 1
            mat = self.materials[0]
        idx = list(self.materials).index(mat)
        self.combo_mat.selectItem(idx)
        #計算結果を取得
        try:
            area = settingDict["area"]
            Iyy = settingDict["Iyy"]
            Izz = settingDict["Izz"]
            Jx = settingDict["Jx"]
            Zyy = settingDict["Zyy"]
            Zzz = settingDict["Zzz"]
        except:
            errFlag = 1
            area, Iyy, Izz, Jx, Zyy, Zzz = "", "", "", "", "", ""
        self.entry_area.set_text(area)
        self.entry_Iyy.set_text(Iyy)
        self.entry_Izz.set_text(Izz)
        self.entry_Jx.set_text(Jx)
        self.label_Zyy.set_text(Zyy)
        self.label_Zzz.set_text(Zzz)
        #丸棒を取得
        try:
            d = settingDict["round"]["d"]
        except:
            errFlag = 1
            d = ""
        self.entryMaru_dia.set_text(d)
        #pipe
        try:
            d = settingDict["pipe"]["d"]
            t = settingDict["pipe"]["t"]
        except:
            errFlag = 1
            d, t = "", ""
        self.entryPipe_d.set_text(d)
        self.entryPipe_t.set_text(t)
        #角棒を取得
        try:
            B = settingDict["square"]["B"]
            h = settingDict["square"]["h"]
        except:
            errFlag = 1
            B, h = "", ""
        self.entryKaku_B.set_text(B)
        self.entryKaku_h.set_text(h)
        #中空角棒
        try:
            B = settingDict["squarePipe"]["B"]
            h = settingDict["squarePipe"]["h"]
            t = settingDict["squarePipe"]["t"]
        except:
            errFlag = 1
            B, h, t = "", "", ""
        self.entryChuKaku_B.set_text(B)
        self.entryChuKaku_h.set_text(h)
        self.entryChuKaku_t.set_text(t)
        #H型
        try:
            B = settingDict["H"]["B"]
            h = settingDict["H"]["h"]
            t1 = settingDict["H"]["t1"]
            t2 = settingDict["H"]["t2"]
        except:
            errFlag = 1
            B, h, t1, t2 = "", "", "", ""
        self.entryH_B.set_text(B)
        self.entryH_h.set_text(h)
        self.entryH_t1.set_text(t1)
        self.entryH_t2.set_text(t2)
        #I型
        try:
            B = settingDict["I"]["B"]
            h = settingDict["I"]["h"]
            t1 = settingDict["I"]["t1"]
            t2 = settingDict["I"]["t2"]
        except:
            errFlag = 1
            B, h, t1, t2 = "", "", "", ""
        self.entryI_B.set_text(B)
        self.entryI_h.set_text(h)
        self.entryI_t1.set_text(t1)
        self.entryI_t2.set_text(t2)
        #L型
        try:
            B = settingDict["L"]["B"]
            h = settingDict["L"]["h"]
            t1 = settingDict["L"]["t1"]
            t2 = settingDict["L"]["t2"]
        except:
            errFlag = 1
            B, h, t1, t2 = "", "", "", ""
        self.entryL_B.set_text(B)
        self.entryL_h.set_text(h)
        self.entryL_t1.set_text(t1)
        self.entryL_t2.set_text(t2)
        #errorチェック
        if errFlag == 1:
            title = _("エラー")
            mess = _("設定内容が読み込めませんでした。")
            self.errDialog(title, mess)
                            

    def errDialog(self, title, message):
        dlg = Gtk.MessageDialog(
                #parent=mainWindow,
                message_type=Gtk.MessageType.ERROR,
                buttons=Gtk.ButtonsType.OK)
        dlg.set_title(title)
        mess1 = message.split(ls)[0]
        mess2 = ls.join(message.split(ls)[1:])
        dlg.set_markup(mess1)
        dlg.format_secondary_text(mess2)
        dlg.run() 
        dlg.destroy()

if __name__ == "__main__":
    import gettext
    gettext.install("app")
    _ = gettext.gettext

    dialog = dialogApp()
    stat = dialog.show()
